;
; PID controller for PIC 16F690 (Multi-CPU)
;
; Licensed under GPLV3+
;
; (c) 2011 James Beau Wilkinson
;
;Up to 31 Scrapnet "stations"; there is a macro for each (MBOT_STAT1,
; MBOT_STAT2, etc.)
;
;Only 2 have a GUI in the code as originally provided; but the necessary
; "hooks" are provided to implement up to 31.
;
#define MBOT_STAT1
;
; "Constants" - each of these is actually a macro that emits an SFP floating
; point number onto the main stack. In practice, the macro definitions below
; serve as program floating point constants.
;
;
; Scaling and Tuning Constants
;
K_SUB_P macro ;(68+128)/128 * (2^5) = 49.0   
 movlw .68 ;P constant - "fine" portion
 PUSH
 movlw .5 ;P constant - "coarse" portion
 PUSH
 endm
K_SUB_I macro
 movlw .32
 PUSH
 movlw .252
 PUSH
 endm
K_SUB_D macro
 movlw .122
 PUSH
 movlw .11
 PUSH
 endm
POS_MAX_I macro ;250/12 * 2^11=4000.0
 movlw .122
 PUSH
 movlw .11
 PUSH
 endm
NEG_MIN_I macro
 movlw .250      ;-4000.0
 PUSH
 movlw .11
 PUSH
 endm
SCALE_SHIFT macro
 movlw .249
 PUSH
 movlw .7
 PUSH
 endm
SCALE_FACTOR macro
 movlw .26
 PUSH
 movlw .252
 PUSH
 endm
MAX_SETPOINT macro ;436.0
 movlw .90
 PUSH
 movlw .8
 PUSH
 endm
MIN_SETPOINT macro ;160/128 * 2^3 = 10.0
 movlw .122
 PUSH
 movlw .7
 PUSH
 endm
INIT_SETPOINT macro
 movlw .23
 PUSH
 movlw .8
 PUSH
 endm
JOYSTICK_RIGHT macro ;Next 4 are joystick limits beyond which mov't is commanded
 movlw .15 ; These range from SFP 0.0 (coarse=-128, fine=0) to SFP 1023.0
 PUSH ; (coarse=9, fine=127)
 movlw .9
 PUSH
 endm
JOYSTICK_LEFT macro
 movlw .92
 PUSH
 movlw .6
 PUSH
 endm
JOYSTICK_UP macro
 movlw .122
 PUSH
 movlw .9
 PUSH
 endm
JOYSTICK_DOWN macro
 movlw .112
 PUSH
 movlw .4
 PUSH
 endm
VGRAPH_SETX macro
 movlw .5
 PUSH
  endm
VGRAPH_POSX macro
 movlw .6
 PUSH
 endm
VGRAPH_Y macro
 movlw .19
 PUSH
 endm
HGRAPH_X macro
 movlw .60
 PUSH
 endm
HGRAPH_SETY macro
 movlw .15
 PUSH
 endm
HGRAPH_POSY macro
 movlw .16
 PUSH
 endm
MAX_GRAPHABLE macro
 movlw .112
 PUSH
 movlw .3
 PUSH
  endm
BARGRAPH_WIDTH macro
 movlw .15
 PUSH
 endm
;
; Other Useful Constants
;
SFP_ONE macro
 movlw .0
 PUSH
 movlw .0
 PUSH
 endm
SFP_NEGATIVE_ONE macro
 movlw .128
 PUSH
 movlw .0
 PUSH
 endm
SFP_ZERO macro
 movlw .0
 PUSH
 movlw .128
 PUSH
 endm
MAX_PWM_COMMAND macro
 movlw .0
 PUSH
 movlw .10
 PUSH
 endm
MIN_PWM_COMMAND macro
 movlw .128
 PUSH
 movlw .10
 PUSH
 endm
POS_CHANNEL_IN macro
 movlw .4
 PUSH
 endm
JOY_CHANNEL_IN macro
 movlw .0
 PUSH
 endm
;Event time is expressed as a real averaged difference in TMR1H, based on
; observation in MPSIM... set it (at 9.0) to reflect the average loop time
; plus the average ISR time. This results in integral and derivative
; approximation code that assumes the values measured near the top of each
; iteration of the loop were in place for the whole duration of the loop
; (including, potentially, a single timer event).
EVENT_TIME macro
 movlw .8
 PUSH
 movlw .4
 PUSH
 endm
;Next constant was tuned to yield a reasonable ET value for the loop iteration
; immediately after the loop iteration where ticked=1. Using a constant eases
; the management of concurrency issues. Otherwise, we might end up passing a
; pre-rollover value, for example, resulting in a botched ET calculation.
T_SUB_ISR macro
 movlw .96
 PUSH
 movlw .3
 PUSH
 endm
;
; Function Prototypes
;
;
; Below is something resembling a system of function prototypes, in the form of
; macro declarations which are then used to effect calculations involving
; parameters in the function bodies... this is not completed for all functions /
; parameters here. CONTROL prototype - here are macros that take each of the
; parameters (SFP or byte, as approrpiate) for the CONTROL function and emits it
; to the top of the main stack. For SFP values, correct byte order is preserved
; (mantissa pushed, then the exponent byte atop it). The macros are ordered,
; below, in the order in which actual parameters to CONTROL end up getting
; pushed at runtime.
CONTROL_TIME_PARMF macro
 movlw .9
 PUSH
 FAR_CALL control,parm
 movlw .8
 PUSH
 FAR_CALL control,parm
 endm
CONTROL_PREV_PARMF macro
 movlw .7
 PUSH
 FAR_CALL control,parm
 movlw .6
 PUSH
 FAR_CALL control,parm
 endm
CONTROL_INTEGRAL_PARMF macro
 movlw .5
 PUSH
 FAR_CALL control,parm
 movlw .4
 PUSH
 FAR_CALL control,parm
 endm
CONTROL_SETP_PARMF macro
 movlw .3
 PUSH
 FAR_CALL control,parm
 movlw .2
 PUSH
 FAR_CALL control,parm
 endm
CONTROL_SAMPLE_PARMF macro
 movlw .1
 PUSH
 FAR_CALL control,parm
 movlw .0
 PUSH
 FAR_CALL control,parm
 endm
;The signature can even extend beyond the base pointer into the auto variable.
; Here, a byte "auto" var and a single "auto" SPF value is allocated by (each
; instance of) the "control" function.
CONTROL_NEWTIME_VARF macro
 movlw -.1
 PUSH
 FAR_CALL control,parm
 movlw -.2
 PUSH
 FAR_CALL control,parm
 endm
CONTROL_RAW_ET_VARF macro
 movlw -.3
 PUSH
 FAR_CALL control,parm
 movlw -.4
 PUSH
 FAR_CALL control,parm
 endm
CONTROL_TICKED_VARB macro
 movlw -.5
 PUSH
 FAR_CALL control,parm
 endm
CONTROL_ET_VARF macro
 movlw -.6
 PUSH
 FAR_CALL control,parm
 movlw -.7
 PUSH
 FAR_CALL control,parm
 endm
CONTROL_RESAMPLE_VARF macro
 movlw -.8
 PUSH
 FAR_CALL control,parm
 movlw -.9
 PUSH
 FAR_CALL control,parm
 endm
CONTROL_DIFF_VARF macro
 movlw -.10
 PUSH
 FAR_CALL control,parm
 movlw -.11
 PUSH
 FAR_CALL control,parm
 endm
CONTROL_ERROR_VARF macro
 movlw -.12
 PUSH
 FAR_CALL control,parm
 movlw -.13
 PUSH
 FAR_CALL control,parm
 endm
CONTROL_SIGMA_VARF macro
 movlw -.14
 PUSH
 FAR_CALL control,parm
 movlw -.15
 PUSH
 FAR_CALL control,parm
 endm
;
OURFTOU_VALUE_PARMF macro
 movlw .1
 PUSH
 FAR_CALL ourftou,parm
 movlw .0
 PUSH
 FAR_CALL ourftou,parm
 endm
;
LONGF_TIME_PARMF macro
 movlw .7
 PUSH
 FAR_CALL longf,parm
 movlw .6
 PUSH
 FAR_CALL longf,parm
 endm
LONGF_LAST_PARMF macro
 movlw .5
 PUSH
 FAR_CALL longf,parm
 movlw .4
 PUSH
 FAR_CALL longf,parm
 endm
LONGF_INTEGRAL_PARMF macro
 movlw .3
 PUSH
 FAR_CALL longf,parm
 movlw .2
 PUSH
 FAR_CALL longf,parm
 endm
LONGF_SETPOINT_PARMF macro
 movlw .1
 PUSH
 FAR_CALL longf,parm
 movlw .0
 PUSH
 FAR_CALL longf,parm
 endm
;
ADJUST_SETPOINT_PARMF macro
 movlw .3
 PUSH
 FAR_CALL adjust,parm
 movlw .2
 PUSH
 FAR_CALL adjust,parm
 endm
ADJUST_SAMPLE_PARMF macro
 movlw .1
 PUSH
 FAR_CALL adjust,parm
 movlw .0
 PUSH
 FAR_CALL adjust,parm
 endm
;
USRPWM_D_PARMF macro
 movlw .5
 PUSH
 FAR_CALL usrpwm,parm
 movlw .4
 PUSH
 FAR_CALL usrpwm,parm
 endm
USRPWM_I_PARMF macro
 movlw .3
 PUSH
 FAR_CALL usrpwm,parm
 movlw .2
 PUSH
 FAR_CALL usrpwm,parm
 endm
USRPWM_P_PARMF macro
 movlw .1
 PUSH
 FAR_CALL usrpwm,parm
 movlw .0
 PUSH
 FAR_CALL usrpwm,parm
 endm
;
CONFORMI_INTEGRAL_PARMF macro
 movlw .1
 PUSH
 FAR_CALL conform_i,parm
 movlw .0
 PUSH
 FAR_CALL conform_i,parm
 endm
;
SAMPLEF_CHANNEL_PARMU macro
 movlw .0
 PUSH
 FAR_CALL samplef,parm
 endm
#include "hloe.inc"
;
PWMF_CHANNEL_PARMU macro
 movlw .0
 PUSH
 FAR_CALL pwmf,parm
 endm
PWMF_COMMAND_PARMF macro
 movlw .2
 PUSH
 FAR_CALL pwmf,parm
 movlw .1
 PUSH
 FAR_CALL pwmf,parm
 endm
;
; C O N F I G
;
 __config (_EXTRC_OSC_NOCLKOUT& _WDT_OFF  & _MCLRE_OFF & _IESO_OFF & _FCMEN_OFF  & _PWRTE_ON & _BOR_OFF)
hllv2setf UDATA
hllsetfl RES .1
#define setf hllsetfl
hllv2setg UDATA
hllsetgl RES .1
#define setg hllsetgl
hllv2ticked UDATA
hlltickedl RES .1
#define ticked hlltickedl
Resetv code 0
 bcf INTCON,GIE
 pagesel hloego
 goto hloego
#ifdef HLLMULTITASK
vectr code 4  ;Device specific
ISR:
 PREEMPT
#endif
 banksel PIR1
 btfss PIR1,TMR1IF
 goto hllnotisr49
 bcf PIR1,TMR1IF
 movlw .1
 banksel ticked
 movwf ticked
 BARGRAPH_WIDTH
 banksel setf
 movfw setf
 PUSH
#ifdef MBOT_STAT0
 VGRAPH_SETX
 VGRAPH_Y
 FAR_CALL ISR,graphy
 movlw .8 ;Backspace
 PUSH
 FAR_CALL ISR,printch
 movlw .8 ;Backspace
 PUSH
 FAR_CALL ISR,printch
 movlw 'S' ;For "Setpoint"
 PUSH
 FAR_CALL ISR,printch
 BARGRAPH_WIDTH
 banksel setg
 movfw setg
 PUSH
 VGRAPH_POSX
 VGRAPH_Y
 FAR_CALL ISR,graphy
 movlw 'P'  ;For "Position"
 PUSH
 FAR_CALL ISR,printch
#endif
#ifdef MBOT_STAT1
 HGRAPH_X
 HGRAPH_SETY
 FAR_CALL ISR,graphx
 movlw 'S' ;For "Setpoint"
 PUSH
 FAR_CALL ISR,printch
 BARGRAPH_WIDTH
 banksel setg
 movfw setg
 PUSH
 HGRAPH_X
 HGRAPH_POSY
 FAR_CALL ISR,graphx
 movlw 'P'   ;For position
 PUSH
 FAR_CALL ISR,printch
#endif
hllnotisr49:
;
; R O B O T
;
; H E A D E R
;
#ifdef HLLMULTITASK
ExitISR:
 RESUME
 retfie
#endif
mainvars udata_shr
#ifdef HLLMULTITASK
pbase res .1   ; Used for parameterized functions, to clean up parms/push args
pbaseisr res .1
#else
pbase res .1
#endif
PC_Save res .1
main code
hloego:
 clrf in_isr
 movlw stack-1  ;Set stack starting position based on literals det. by linker
 movwf FSR
 movlw alt_stack-1
 movwf alt_fsr
 clrf softstack0
 bankisel stack
 pagesel hlluserprog
 goto hlluserprog
hllupuser CODE
hlluserprog:
 FAR_CALL hlluserprog,main_setup
 movlw TMR1ON
 PUSH
 banksel T1CON
 movfw T1CON
 PUSH
 FAR_CALL hlluserprog,clearbit
 POP
 banksel T1CON
 movwf T1CON
 FAR_CALL hlluserprog,wait_button
 movlw .0
 banksel TMR1H
 movwf TMR1H
 movlw .0
 banksel TMR1L
 movwf TMR1L
 movlw GIE
 PUSH
 banksel INTCON
 movfw INTCON
 PUSH
 FAR_CALL hlluserprog,setbit
 POP
 banksel INTCON
 movwf INTCON
 movlw .0
 banksel ticked
 movwf ticked
;
;SCRAPNET station-specific timing
;
#ifdef MBOT_STAT0
 movlw .0 ;Assumes up to 32 stations
#endif
#ifdef MBOT_STAT1
 movlw .8
#endif
#ifdef MBOT_STAT2
 movlw .16
#endif
#ifdef MBOT_STAT3
 movlw .24
#endif
#ifdef MBOT_STAT4
 movlw .32
#endif
#ifdef MBOT_STAT5
 movlw .40
#endif
#ifdef MBOT_STAT6
 movlw .48
#endif
#ifdef MBOT_STAT7
 movlw .56
#endif
#ifdef MBOT_STAT8
 movlw .64
#endif
#ifdef MBOT_STAT9
 movlw .72
#endif
#ifdef MBOT_STAT10
 movlw .80
#endif
#ifdef MBOT_STAT11
 movlw .88
#endif
#ifdef MBOT_STAT12
 movlw .96
#endif
#ifdef MBOT_STAT13
 movlw .104
#endif
#ifdef MBOT_STAT14
 movlw .112
#endif
#ifdef MBOT_STAT15
 movlw .120
#endif
#ifdef MBOT_STAT16
 movlw .128
#endif
#ifdef MBOT_STAT17
 movlw .136
#endif
#ifdef MBOT_STAT18
 movlw .144
#endif
#ifdef MBOT_STAT19
 movlw .152
#endif
#ifdef MBOT_STAT20
 movlw .160
#endif
#ifdef MBOT_STAT21
 movlw .168
#endif
#ifdef MBOT_STAT22
 movlw .176
#endif
#ifdef MBOT_STAT23
 movlw .184
#endif
#ifdef MBOT_STAT24
 movlw .192
#endif
#ifdef MBOT_STAT25
 movlw .200
#endif
#ifdef MBOT_STAT26
 movlw .208
#endif
#ifdef MBOT_STAT27
 movlw .216
#endif
#ifdef MBOT_STAT28
 movlw .224
#endif
#ifdef MBOT_STAT29
 movlw .232
#endif
#ifdef MBOT_STAT30
 movlw .240
#endif
#ifdef MBOT_STAT31
 movlw .248
#endif
 banksel TMR1H
 movwf TMR1H
 movlw TMR1ON
 PUSH
 banksel T1CON
 movfw T1CON
 PUSH
 FAR_CALL hlluserprog,setbit
 POP
 banksel T1CON
 movwf T1CON
 SFP_ZERO
 SFP_ONE
 POS_CHANNEL_IN
 FAR_CALL hlluserprog,samplef
 FAR_CALL hlluserprog,addf
 SFP_ZERO
 INIT_SETPOINT
 pagesel longf
 goto longf
hllprogend:
 goto hllprogend
hllt450 CODE
ourftou:
 movf FSR,w
 FAR_CALL ourftou , kpush
 OURFTOU_VALUE_PARMF
 MAX_GRAPHABLE
 FAR_CALL ourftou,gtf
 POP
 xorlw .0
 btfsc STATUS,Z
 goto hlllb51J3
 BARGRAPH_WIDTH
 goto hlllb51J4
hlllb51J3:
 SFP_ZERO
 OURFTOU_VALUE_PARMF
 FAR_CALL ourftou,gtf
 POP
 xorlw .0
 btfsc STATUS,Z
 goto hlllb51J5
 movlw .0
 PUSH
 goto hlllb51J6
hlllb51J5:
 FAR_CALL ourftou,ftou
hlllb51J6:
hlllb51J4:
 FAR_CALL ourftou,kpop
#ifdef HLLMULTITASK
 movf in_isr,f ;In shared RAM, no need for BANKSEL
 btfsc STATUS,Z
 goto $+3
 movwf pbase+1 ;in_isr... the "in isr" clause comes first in all of these macros,
 goto $+2      ; because isr execution is preemptive, and returns only at the
 movwf pbase   ; completion of all triggered routines. So, it can starve out the 
#else         ; main thread, and speeding up isr execution helps this problem. 
 movwf pbase   ; In most of these macros, in-isr execution takes just 4 
#endif        ; instruction cycles.
 POP
 FAR_CALL ourftou , kpush
#ifdef HLLMULTITASK
 movf in_isr,f ;In shared RAM, no need for BANKSEL
 btfsc STATUS,Z
 goto $+3
 movf pbase+1,w   ;in_isr
 goto $+2
 movf pbase,w ;not in_isr
#else
 movf pbase,w
#endif
;LOAD POINTER
 movwf FSR
 DISCARD
 DISCARD
 FAR_CALL ourftou , kpop
 PUSH
 return
hllt455 CODE
longf:
 movf FSR,w
 FAR_CALL longf , kpush
 SCALE_SHIFT
 LONGF_SETPOINT_PARMF
 FAR_CALL longf,addf
 SCALE_FACTOR
 FAR_CALL longf,mulf
 FAR_CALL longf,ourftou
 POP
 banksel setf
 movwf setf
 SCALE_SHIFT
 LONGF_LAST_PARMF
 FAR_CALL longf,addf
 SCALE_FACTOR
 FAR_CALL longf,mulf
 FAR_CALL longf,ourftou
 POP
 banksel setg
 movwf setg
 movlw .0
 PUSH
 movlw .0
 PUSH
 POS_CHANNEL_IN
 FAR_CALL longf,samplef
 FAR_CALL longf,addf
 FAR_CALL longf,control
 JOY_CHANNEL_IN
 FAR_CALL longf,samplef
 FAR_CALL longf,adjust
 KDISCARD
 goto longf
 KDISCARD
 return
hllt456 CODE
adjust:
 movf FSR,w
 FAR_CALL adjust , kpush
 ADJUST_SAMPLE_PARMF
#ifdef MBOT_STAT0
 JOYSTICK_UP
 FAR_CALL adjust,gtf
 ADJUST_SETPOINT_PARMF
 MIN_SETPOINT
 FAR_CALL adjust,gtf
 FAR_CALL adjust,andb
 POP
 xorlw .0
 btfsc STATUS,Z
 goto dlllb51J6
 movlw .128
#endif
#ifdef MBOT_STAT1
 JOYSTICK_RIGHT
 FAR_CALL adjust,gtf
 MAX_SETPOINT
 ADJUST_SETPOINT_PARMF
 FAR_CALL adjust,gtf
 FAR_CALL adjust,andb
 POP
 xorlw .0
 btfsc STATUS,Z
 goto dlllb51J6
 movlw .0
#endif
 PUSH
 movlw .0
 PUSH
 ADJUST_SETPOINT_PARMF
 FAR_CALL adjust,addf
 goto dlllb51J7
dlllb51J6:
#ifdef MBOT_STAT0
 JOYSTICK_DOWN
 ADJUST_SAMPLE_PARMF
 FAR_CALL adjust,gtf
 MAX_SETPOINT
 ADJUST_SETPOINT_PARMF
 FAR_CALL adjust,gtf
 FAR_CALL adjust,andb
 POP
 xorlw .0
 btfsc STATUS,Z
 goto dlllb51J8
 movlw .0
 PUSH
#endif
#ifdef MBOT_STAT1
 JOYSTICK_LEFT
 ADJUST_SAMPLE_PARMF
 FAR_CALL adjust,gtf
 ADJUST_SETPOINT_PARMF
 MIN_SETPOINT
 FAR_CALL adjust,gtf
 FAR_CALL adjust,andb
 POP
 xorlw .0
 btfsc STATUS,Z
 goto dlllb51J8
 movlw .128
 PUSH
#endif
 movlw .0
 PUSH
 ADJUST_SETPOINT_PARMF
 FAR_CALL adjust,addf
 goto dlllb51J9
dlllb51J8:
 ADJUST_SETPOINT_PARMF
dlllb51J9:
dlllb51J7:
 FAR_CALL adjust,kpop
#ifdef HLLMULTITASK
 movf in_isr,f
 btfsc STATUS,Z
 goto $+3
 movwf pbase+1
 goto $+2
 movwf pbase
#else
 movwf pbase
#endif
 POP
 FAR_CALL adjust , kpush
 POP
 FAR_CALL adjust , kpush
#ifdef HLLMULTITASK
 movf in_isr,f
 btfsc STATUS,Z
 goto $+3
 movf pbase+1,w ;in_isr
 goto $+2
 movf pbase,w ;not in_isr
#else
 movf pbase,w
#endif
;LOAD POINTER
 movwf FSR
 DISCARD
 DISCARD
 DISCARD
 DISCARD
 FAR_CALL adjust , kpop
 PUSH
 FAR_CALL adjust , kpop
 PUSH
 return
hllt4511 CODE
usrpwm:
 movf FSR,w
 FAR_CALL usrpwm , kpush
 K_SUB_P
 USRPWM_P_PARMF
 FAR_CALL usrpwm,mulf
 K_SUB_I
 USRPWM_I_PARMF
 FAR_CALL usrpwm,mulf
 K_SUB_D
 USRPWM_D_PARMF
 FAR_CALL usrpwm,mulf
 FAR_CALL usrpwm,addf
 FAR_CALL usrpwm,addf
 FAR_CALL usrpwm,copyf
 FAR_CALL usrpwm,copyf
 SFP_ZERO
 FAR_CALL usrpwm,gtf
 POP
 xorlw .0
 btfsc STATUS,Z
 goto hlllb51J14
 MAX_PWM_COMMAND
 FAR_CALL usrpwm,gtf
 POP
 xorlw .0
 btfsc STATUS,Z
 goto hlllb51J16
 MAX_PWM_COMMAND
 movlw .1 ;Channel number
 PUSH
 FAR_CALL usrpwm,pwmf
 goto hlllb51J17
hlllb51J16:
 movlw .1 ;Channel number
 PUSH
 FAR_CALL usrpwm,pwmf
hlllb51J17:
 goto hlllb51J15
hlllb51J14:
 MIN_PWM_COMMAND
 FAR_CALL usrpwm,gtf
 POP
 xorlw .0
 btfsc STATUS,Z
 goto hlllb51J18
 SFP_NEGATIVE_ONE
 FAR_CALL usrpwm,mulf
 movlw .0 ;Channel number
 PUSH
 FAR_CALL usrpwm,pwmf
 goto hlllb51J19
hlllb51J18:
 MAX_PWM_COMMAND
 movlw .0 ;Channel number
 PUSH
 FAR_CALL usrpwm,pwmf
hlllb51J19:
hlllb51J15:
 FAR_CALL usrpwm,kpop
 ;LOAD POINTER
 movwf FSR
 DISCARD
 DISCARD
 DISCARD
 DISCARD
 DISCARD
 DISCARD
 return
hllt4518 CODE
conform_i:
 movf FSR,w
 FAR_CALL conform_i , kpush
 CONFORMI_INTEGRAL_PARMF
 SFP_ZERO
 FAR_CALL conform_i,gtf
 POP
 xorlw .0
 btfsc STATUS,Z
 goto cnflb51J18
 CONFORMI_INTEGRAL_PARMF
 POS_MAX_I
 FAR_CALL conform_i,gtf
 POP
 xorlw .0
 btfsc STATUS,Z
 goto cnflb51J20
 POS_MAX_I
 goto cnflb51J21
cnflb51J20:
cnflb51J21:
 goto cnflb51J19
cnflb51J18:
 NEG_MIN_I
 CONFORMI_INTEGRAL_PARMF
 FAR_CALL conform_i,gtf
 POP
 xorlw .0
 btfsc STATUS,Z
 goto cnflb51J22
 NEG_MIN_I
 goto cnflb51J23
cnflb51J22:
cnflb51J23:
cnflb51J19:
 FAR_CALL conform_i,kpop
#ifdef HLLMULTITASK
 movf in_isr,f
 btfsc STATUS,Z
 goto $+3
 movwf pbase+1
 goto $+2
 movwf pbase
#else
 movwf pbase
#endif
 POP
 FAR_CALL conform_i , kpush
 POP
 FAR_CALL conform_i , kpush
#ifdef HLLMULTITASK
 movf in_isr,f
 btfsc STATUS,Z
 goto $+3
 movf pbase+1,w ;in_isr
 goto $+2
 movf pbase,w ;not in_isr
#else
 movf pbase,w
#endif
;LOAD POINTER
 movwf FSR
 DISCARD
 DISCARD
 FAR_CALL conform_i , kpop
 PUSH
 FAR_CALL conform_i , kpop
 PUSH
 return
hllt4525 CODE
control:
 movf FSR,w
 FAR_CALL control , kpush
 banksel TMR1H
 movfw TMR1H
 PUSH
 FAR_CALL control,utof
 CONTROL_NEWTIME_VARF
 SFP_NEGATIVE_ONE
 CONTROL_TIME_PARMF
 FAR_CALL control,mulf
 FAR_CALL control,addf
 banksel ticked
 movfw ticked
 xorlw .0
 btfsc STATUS,Z
 goto notick
 movlw .1
 PUSH
 movlw .0
 banksel ticked
 movwf ticked
 goto ctllb51J26
notick:
 movlw .0
 PUSH
ctllb51J26:
 CONTROL_TICKED_VARB
 POP
 xorlw .0
 btfsc STATUS,Z
 goto ctllb51J27
 EVENT_TIME
 goto ctllb51J28
ctllb51J27:
 CONTROL_RAW_ET_VARF
ctllb51J28:
 CONTROL_TICKED_VARB
 POP
 xorlw .0
 btfsc STATUS,Z
 goto ctllb51J29
 SFP_ONE
 POS_CHANNEL_IN
 FAR_CALL control,samplef
 FAR_CALL control,addf
 goto ctllb51J30
ctllb51J29:
 CONTROL_SAMPLE_PARMF
ctllb51J30:
 CONTROL_PREV_PARMF
 SFP_NEGATIVE_ONE
 CONTROL_RESAMPLE_VARF
 FAR_CALL control,mulf
 FAR_CALL control,addf
 CONTROL_SETP_PARMF
 SFP_NEGATIVE_ONE
 CONTROL_RESAMPLE_VARF
 FAR_CALL control,mulf
 FAR_CALL control,addf
 CONTROL_INTEGRAL_PARMF
 CONTROL_ERROR_VARF
 CONTROL_ET_VARF
 FAR_CALL control,mulf
 FAR_CALL control,addf
 FAR_CALL control,conform_i
 CONTROL_DIFF_VARF
 FAR_CALL control,iszerof
 POP
 xorlw .0
 btfsc STATUS,Z
 goto ctllb51J31
 SFP_ZERO
 goto ctllb51J32
ctllb51J31:
 CONTROL_DIFF_VARF
 CONTROL_ET_VARF
 FAR_CALL control,divf
ctllb51J32:
 CONTROL_SIGMA_VARF
 CONTROL_ERROR_VARF
 FAR_CALL control,usrpwm
 CONTROL_TICKED_VARB
 POP
 xorlw .0
 btfsc STATUS,Z
 goto ctllb51J33
 T_SUB_ISR
 goto ctllb51J34
ctllb51J33:
 CONTROL_NEWTIME_VARF
ctllb51J34:
 CONTROL_RESAMPLE_VARF
 CONTROL_SIGMA_VARF
 CONTROL_SETP_PARMF
 FAR_CALL control,kpop
#ifdef HLLMULTITASK
 movf in_isr,f
 btfsc STATUS,Z
 goto $+3
 movwf pbase+1
 goto $+2
 movwf pbase
#else
 movwf pbase
#endif
 POP
 FAR_CALL control , kpush
 POP
 FAR_CALL control , kpush
 POP
 FAR_CALL control , kpush
 POP
 FAR_CALL control , kpush
 POP
 FAR_CALL control , kpush
 POP
 FAR_CALL control , kpush
 POP
 FAR_CALL control , kpush
 POP
 FAR_CALL control , kpush
#ifdef HLLMULTITASK
 movf in_isr,f
 btfsc STATUS,Z
 goto $+3
 movf pbase+1,w ;in_isr
 goto $+2
 movf pbase,w ;not in_isr
#else
 movf pbase,w
#endif
 ;LOAD POINTER
 movwf FSR
 DISCARD
 DISCARD
 DISCARD
 DISCARD
 DISCARD
 DISCARD
 DISCARD
 DISCARD
 DISCARD
 DISCARD
 FAR_CALL control , kpop
 PUSH
 FAR_CALL control , kpop
 PUSH
 FAR_CALL control , kpop
 PUSH
 FAR_CALL control , kpop
 PUSH
 FAR_CALL control , kpop
 PUSH
 FAR_CALL control , kpop
 PUSH
 FAR_CALL control , kpop
 PUSH
 FAR_CALL control , kpop
 PUSH
 return
hllt4536 CODE
wait_button:
 banksel PORTA
 movfw PORTA
 PUSH
 movlw .16
 PUSH
 FAR_CALL wait_button,andu
 POP
 xorlw .0
 btfsc STATUS,Z
 goto hlllb51J39
 goto wait_button
 goto hlllb51J40
hlllb51J39:
hlllb51J40:
 return
hllt4539 CODE
main_setup:
 movlw .0
 banksel setf
 movwf setf
 movlw .0
 banksel setg
 movwf setg
 movlw .0
 banksel INTCON
 movwf INTCON
 movlw .0
 banksel ANSELH
 movwf ANSELH
 movlw .0
 banksel ANSEL
 movwf ANSEL
 FAR_CALL main_setup,pwmon
 movlw SYNC
 PUSH
 movlw BRGH
 PUSH
 banksel TXSTA
 movfw TXSTA
 PUSH
 FAR_CALL main_setup,setbit
 FAR_CALL main_setup,clearbit
 POP
 banksel TXSTA
 movwf TXSTA
 movlw BRG16
 PUSH
 movlw SCKP
 PUSH
 banksel BAUDCTL
 movfw BAUDCTL
 PUSH
 FAR_CALL main_setup,setbit
 FAR_CALL main_setup,setbit
 POP
 banksel BAUDCTL
 movwf BAUDCTL
 movlw CREN
 PUSH
 banksel RCSTA
 movfw RCSTA
 PUSH
 FAR_CALL main_setup,clearbit
 POP
 banksel RCSTA
 movwf RCSTA
 movlw CREN
 PUSH
 banksel RCSTA
 movfw RCSTA
 PUSH
 FAR_CALL main_setup,setbit
 POP
 banksel RCSTA
 movwf RCSTA
 movlw .25
 banksel SPBRG
 movwf SPBRG
 movlw .0
 banksel SPBRGH
 movwf SPBRGH
 movlw TXEN
 PUSH
 movlw TX9
 PUSH
 banksel TXSTA
 movfw TXSTA
 PUSH
 FAR_CALL main_setup,clearbit
 FAR_CALL main_setup,setbit
 POP
 banksel TXSTA
 movwf TXSTA
 movlw SPEN
 PUSH
 banksel RCSTA
 movfw RCSTA
 PUSH
 FAR_CALL main_setup,setbit
 POP
 banksel RCSTA
 movwf RCSTA
 movlw TMR1CS
 PUSH
 movlw TMR1GE
 PUSH
 movlw T1CKPS1
 PUSH
 movlw T1CKPS0
 PUSH
 movlw NOT_T1SYNC
 PUSH
 movlw T1OSCEN
 PUSH
 banksel T1CON
 movfw T1CON
 PUSH
 FAR_CALL main_setup,clearbit
 FAR_CALL main_setup,clearbit
 FAR_CALL main_setup,setbit
 FAR_CALL main_setup,setbit
 FAR_CALL main_setup,clearbit
 FAR_CALL main_setup,clearbit
 POP
 banksel T1CON
 movwf T1CON
 movlw TMR1IE
 PUSH
 banksel PIE1
 movfw PIE1
 PUSH
 FAR_CALL main_setup,setbit
 POP
 banksel PIE1
 movwf PIE1
 movlw PEIE
 PUSH
 banksel INTCON
 movfw INTCON
 PUSH
 FAR_CALL main_setup,setbit
 POP
 banksel INTCON
 movwf INTCON
 return
hllt4540 CODE
samplef:
 movf FSR,w
 FAR_CALL samplef , kpush
 movlw .6
 PUSH
 movlw .5
 PUSH
 movlw .4
 PUSH
 banksel ADCON1
 movfw ADCON1
 PUSH
 FAR_CALL samplef,setbit
 FAR_CALL samplef,clearbit
 FAR_CALL samplef,setbit
 POP
 banksel ADCON1
 movwf ADCON1
 SAMPLEF_CHANNEL_PARMU
 movlw .0
 PUSH
 FAR_CALL samplef,eq
 POP
 xorlw .0
 btfsc STATUS,Z
 goto hlllb51J43
 movlw .2
 PUSH
 movlw .3
 PUSH
 movlw .4
 PUSH
 banksel ADCON0
 movfw ADCON0
 PUSH
 FAR_CALL samplef,clearbit
 FAR_CALL samplef,clearbit
 FAR_CALL samplef,clearbit
 goto hlllb51J44
hlllb51J43:
 movlw .2
 PUSH
 movlw .3
 PUSH
 movlw .4
 PUSH
 banksel ADCON0
 movfw ADCON0
 PUSH
 FAR_CALL samplef,setbit
 FAR_CALL samplef,clearbit
 FAR_CALL samplef,clearbit
hlllb51J44:
 POP
 banksel ADCON0
 movwf ADCON0
 FAR_CALL samplef,dispose
 movlw .1
 PUSH
 movlw .7
 PUSH
 movlw .0
 PUSH
 banksel ADCON0
 movfw ADCON0
 PUSH
 FAR_CALL samplef,setbit
 FAR_CALL samplef,setbit
 FAR_CALL samplef,clearbit
 POP
 banksel ADCON0
 movwf ADCON0
 movlw .0
 PUSH
 banksel ANSEL
 movfw ANSEL
 PUSH
 FAR_CALL samplef,setbit
 POP
 banksel ANSEL
 movwf ANSEL
 movlw .1
 PUSH
 banksel ADCON0
 movfw ADCON0
 PUSH
 FAR_CALL samplef,setbit
 POP
 banksel ADCON0
 movwf ADCON0
 FAR_CALL samplef,ourwait
 movlw .0
 PUSH
 movlw .8
 PUSH
 movlw .3
 PUSH
 banksel ADRESH
 movfw ADRESH
 PUSH
 FAR_CALL samplef,andu
 FAR_CALL samplef,utof
 FAR_CALL samplef,mulf
 banksel ADRESL
 movfw ADRESL
 PUSH
 FAR_CALL samplef,utof
 FAR_CALL samplef,addf
 FAR_CALL samplef,kpop
#ifdef HLLMULTITASK
 movf in_isr,f ;In shared RAM, no need for BANKSEL
 btfsc STATUS,Z
 goto $+3
 movwf pbase+1 
 goto $+2      
 movwf pbase   
#else         
 movwf pbase   
#endif
 POP
 FAR_CALL samplef,kpush
 POP
 FAR_CALL samplef,kpush
#ifdef HLLMULTITASK
 movf in_isr,f ;In shared RAM, no need for BANKSEL
 btfsc STATUS,Z
 goto $+3
 movf pbase+1,w   ;in_isr
 goto $+2
 movf pbase,w ;not in_isr
#else
 movf pbase,w
#endif
 ;LOAD POINTER
 movwf FSR
 DISCARD
 FAR_CALL samplef,kpop
 PUSH
 FAR_CALL samplef,kpop
 PUSH
 return
hllt4543 CODE
ourwait:
 movlw .2
 PUSH
 banksel ADCON0
 movfw ADCON0
 PUSH
 FAR_CALL ourwait,andu
 POP
 xorlw .0
 btfsc STATUS,Z
 goto hlllb51J46
 goto ourwait
 goto hlllb51J47
hlllb51J46:
hlllb51J47:
 return
hllt4546 CODE
pwmon:
 movlw .255
 banksel PR2
 movwf PR2
 movlw .2
 PUSH
 movlw .3
 PUSH
 movlw .0
 PUSH
 FAR_CALL pwmon,setbit
 FAR_CALL pwmon,setbit
 POP
 banksel CCP1CON
 movwf CCP1CON
 movlw .0
 banksel CCPR1L
 movwf CCPR1L
 movlw .1
 PUSH
 banksel PIR1
 movfw PIR1
 PUSH
 FAR_CALL pwmon,clearbit
 POP
 banksel PIR1
 movwf PIR1
 movlw .0
 PUSH
 movlw .1
 PUSH
 banksel T2CON
 movfw T2CON
 PUSH
 FAR_CALL pwmon,clearbit
 FAR_CALL pwmon,clearbit
 POP
 banksel T2CON
 movwf T2CON
 movlw .2
 PUSH
 banksel T2CON
 movfw T2CON
 PUSH
 FAR_CALL pwmon,setbit
 POP
 banksel T2CON
 movwf T2CON
 movlw .1
 PUSH
 banksel PIR1
 movfw PIR1
 PUSH
 FAR_CALL pwmon,clearbit
 POP
 banksel PIR1
 movwf PIR1
 FAR_CALL pwmon,pwmin
 movlw .5
 PUSH
 movlw .4
 PUSH
 banksel TRISC
 movfw TRISC
 PUSH
 FAR_CALL pwmon,clearbit
 FAR_CALL pwmon,clearbit
 POP
 banksel TRISC
 movwf TRISC
 movlw .0
 banksel PORTC
 movwf PORTC
 return
hllt4547 CODE
pwmf:
 movf FSR,w
 FAR_CALL pwmf,kpush
 PWMF_CHANNEL_PARMU
 POP
 xorlw .0
 btfsc STATUS,Z
 goto hlllb51J50
 movlw .0
 PUSH
 movlw .1
 PUSH
 banksel PSTRCON
 movfw PSTRCON
 PUSH
 FAR_CALL pwmf,clearbit
 FAR_CALL pwmf,setbit
 POP
 banksel PSTRCON
 movwf PSTRCON
 goto hlllb51J51
hlllb51J50:
 movlw .1
 PUSH
 movlw .0
 PUSH
 banksel PSTRCON
 movfw PSTRCON
 PUSH
 FAR_CALL pwmf,clearbit
 FAR_CALL pwmf,setbit
 POP
 banksel PSTRCON
 movwf PSTRCON
hlllb51J51:
 PWMF_COMMAND_PARMF
 FAR_CALL pwmf,iszerof
 POP
 xorlw .0
 btfsc STATUS,Z
 goto hlllb51J52
 movlw .0
 PUSH
 goto hlllb51J53
hlllb51J52:
 PWMF_COMMAND_PARMF
 movlw .122
 PUSH
 movlw .9
 PUSH
 FAR_CALL pwmf,gtf
 POP
 xorlw .0
 btfsc STATUS,Z
 goto hlllb51J54
 movlw .255
 PUSH
 goto hlllb51J55
hlllb51J54:
 PWMF_COMMAND_PARMF
 movlw .0
 PUSH
 movlw .2
 PUSH
 FAR_CALL pwmf,divf
 FAR_CALL pwmf,ftou
hlllb51J55:
hlllb51J53:
 POP
 banksel CCPR1L
 movwf CCPR1L
 PWMF_COMMAND_PARMF
 movlw .128
 PUSH
 movlw .0
 PUSH
 movlw .0
 PUSH
 movlw .2
 PUSH
 banksel CCPR1L
 movfw CCPR1L
 PUSH
 FAR_CALL pwmf,utof
 FAR_CALL pwmf,mulf
 FAR_CALL pwmf,mulf
 FAR_CALL pwmf,addf
 FAR_CALL pwmf,copyf
 movlw .32
 PUSH
 movlw .1
 PUSH
 FAR_CALL pwmf,gtf
 POP
 xorlw .0
 btfsc STATUS,Z
 goto hlllb51J56
 movlw DC1B1
 PUSH
 movlw DC1B0
 PUSH
 banksel CCP1CON
 movfw CCP1CON
 PUSH
 FAR_CALL pwmf,setbit
 FAR_CALL pwmf,setbit
 POP
 banksel CCP1CON
 movwf CCP1CON
 goto hlllb51J57
hlllb51J56:
 FAR_CALL pwmf,copyf
 movlw .64
 PUSH
 movlw .0
 PUSH
 FAR_CALL pwmf,gtf
 POP
 xorlw .0
 btfsc STATUS,Z
 goto hlllb51J58
 movlw DC1B1
 PUSH
 banksel CCP1CON
 movfw CCP1CON
 PUSH
 FAR_CALL pwmf,setbit
 POP
 banksel CCP1CON
 movwf CCP1CON
 goto hlllb51J59
hlllb51J58:
 FAR_CALL pwmf,copyf
 movlw .0
 PUSH
 movlw .255
 PUSH
 FAR_CALL pwmf,gtf
 POP
 xorlw .0
 btfsc STATUS,Z
 goto hlllb51J60
 movlw DC1B0
 PUSH
 banksel CCP1CON
 movfw CCP1CON
 PUSH
 FAR_CALL pwmf,setbit
 POP
 banksel CCP1CON
 movwf CCP1CON
 goto hlllb51J61
hlllb51J60:
hlllb51J61:
hlllb51J59:
hlllb51J57:
 FAR_CALL pwmf,kpop
 ;LOAD POINTER
 movwf FSR
 DISCARD
 DISCARD
 DISCARD
 return
hllt4560 CODE
pwmin:
 banksel PIR1
 movfw PIR1
 PUSH
 movlw .2
 PUSH
 FAR_CALL pwmin,andu
 POP
 xorlw .0
 btfsc STATUS,Z
 goto hlllb51J63
 goto hlllb51J64
hlllb51J63:
 goto pwmin
hlllb51J64:
 return
hllprgen2:
 goto hllprgen2
hlog1g CODE
andb:
 pagesel mul
 goto mul
hlog1h CODE
andu:
 POP
 andwf HLINDF,w
 decf HLFSR,f
 PUSH
 return
ansiadt udata
aart00 RES .1
aart01 RES .1
aart02 RES .1
ansiara CODE
ansiadu udata
aartpr00 RES .1
cprntuc CODE
#define margpi aartpr00
printu:
 POP
 banksel margpi
 movwf margpi
 PUSH
 movlw .100
 PUSH
 FAR_CALL printu,divu
 POP
 addlw '0'
 PUSH
 FAR_CALL printu, printch
 banksel margpi
 movfw margpi
 PUSH
 movlw .100
 PUSH
 FAR_CALL printu, modu
 movlw .10
 PUSH
 FAR_CALL printu, divu
 POP
 addlw '0'
 PUSH
 FAR_CALL printu, printch
 banksel margpi
 movfw margpi
 PUSH
 movlw .10
 PUSH
 FAR_CALL printu, modu
 POP
 addlw '0'
 PUSH
 FAR_CALL printu, printch
 return
#undefine margpi
#ifdef MBOT_STAT0
ansiaff CODE
#define flgg3 aart00
#define vert aart01
#define cont aart02
graphy:
 movf HLFSR,w
 pagesel kpush
 call kpush
 movlw .2
 PUSH
 FAR_CALL graphy,parm
 POP
 banksel cont
 movwf cont
 movlw .0
 xorwf cont,w
 btfsc STATUS,Z
 goto hllGLBLA
hllELBLA:
 bcf flgg3,0
 goto hllKLBLA
hllFLBLA:
 movlw 0xDB
 PUSH
 FAR_CALL graphy,printch
 banksel cont
 decfsz cont,F
 goto hllELBLA
hllGLBLA:
 movlw .3
 PUSH
 pagesel parm
 call parm
 movlw .2
 PUSH
 pagesel parm
 call parm
 pagesel negti
 call negti
 FAR_CALL graphy,add
 POP
 banksel cont
 movwf cont
 movf cont,f
 btfsc STATUS,Z
 goto hllJLBLA
hllHLBLA:
 bsf flgg3,0
 goto hllKLBLA
hllILBLA:
 movlw 0xB0
 PUSH
 FAR_CALL graphy,printch
 banksel cont
 decfsz cont,F
 goto hllHLBLA
hllJLBLA:
 pagesel kpop
 goto kpop
 movwf HLFSR
 return
hllKLBLA:
 movlw 0x1b
 PUSH
 pagesel printch
 call printch
 movlw '['
 PUSH
 pagesel printch
 call printch
 COPY
 pagesel printu
 call printu
 decf HLINDF,f
 movlw ';'
 PUSH
 pagesel printch
 call printch
 movlw .1
 PUSH
 pagesel parm
 call parm
 pagesel printu
 call printu
 movlw 'H'
 PUSH
 FAR_CALL graphy,printch
 banksel cont
 btfss flgg3,0
 goto hllFLBLA
 goto hllILBLA
#undefine flgg3
#undefine vert
#undefine cont
#endif
#ifdef MBOT_STAT1
ansiard CODE
#define charx aart00
#define ansiG aart01
graphx:
 movlw 0x1b
 PUSH
 FAR_CALL graphx,printch
 movlw '['
 PUSH
 FAR_CALL graphx,printch
 FAR_CALL graphx,printu
 movlw ';'
 PUSH
 FAR_CALL graphx,printch
 FAR_CALL graphx,printu
 movlw 'H'
 PUSH
 FAR_CALL graphx,printch
 POP
 banksel charx
 movwf charx
 xorlw .0
 btfsc STATUS,Z
 goto hllBLBLA
 banksel ansiG
 movwf ansiG
hllLBLAA:
 movlw 0xDB
 PUSH
 FAR_CALL graphx,printch
 banksel ansiG
 decfsz ansiG,f
 goto hllLBLAA
hllBLBLA:
 banksel charx
 movf charx,w
 subwf HLINDF,f
 POP
 xorlw .0
 btfsc STATUS,Z
 goto hllDLBLA
 banksel ansiG
 movwf ansiG
hllCLBLA:
 movlw 0xB0
 PUSH
 FAR_CALL graphx,printch
 banksel ansiG
 decfsz ansiG,f
 goto hllCLBLA
hllDLBLA:
 return
#undefine charx
#undefine ansiG
#endif
hlog1a CODE
#define margp2 hllblss00
clearbit:
#ifdef HLLMULTITASK
#ifdef HLLGUARDhllblss
 movf in_isr,f
 btfsc STATUS,Z
 goto clearbit0
 POP
 banksel margp2
 movwf margp2
 POP
 xorlw .0
 btfss STATUS,Z
 goto tff00h
 movfw margp2
 andlw .255-.1
 goto tff99h
tff00h:
 xorlw .1
 btfss STATUS,Z
 goto tff01h
 movfw margp2
 andlw .255-.2
 goto tff99h
tff01h:
 xorlw .1
 xorlw .2
 btfss STATUS,Z
 goto tff02h
 movfw margp2
 andlw .255-.4
 goto tff99h
tff02h:
 xorlw .2
 xorlw .3
 btfss STATUS,Z
 goto tff03h
 movfw margp2
 andlw .255-.8
 goto tff99h
tff03h:
 xorlw .3
 xorlw .4
 btfss STATUS,Z
 goto tff04h
 movfw margp2
 andlw .255-.16
 goto tff99h
tff04h:
 xorlw .4
 xorlw .5
 btfss STATUS,Z
 goto tff05h
 movfw margp2
 andlw .255-.32
 goto tff99h
tff05h:
 xorlw .5
 xorlw .6
 btfss STATUS,Z
 goto tff07h
 movfw margp2
 andlw .255-.64
 goto tff99h
tff07h:
 movfw margp2
 andlw .255-.128
tff99h:
 PUSH
 return
#undefine margp2
#define margp2 hllblss00+1
clearbit0:
#endif
#endif
 POP
 banksel margp2
 movwf margp2
 POP
 xorlw .0
 btfss STATUS,Z
 goto hllLLBLA
 movfw margp2
 andlw .255-.1
 goto hllSLBLA
hllLLBLA:
 xorlw .1
 btfss STATUS,Z
 goto hllMLBLA
 movfw margp2
 andlw .255-.2
 goto hllSLBLA
hllMLBLA:
 xorlw .1
 xorlw .2
 btfss STATUS,Z
 goto hllNLBLA
 movfw margp2
 andlw .255-.4
 goto hllSLBLA
hllNLBLA:
 xorlw .2
 xorlw .3
 btfss STATUS,Z
 goto hllOLBLA
 movfw margp2
 andlw .255-.8
 goto hllSLBLA
hllOLBLA:
 xorlw .3
 xorlw .4
 btfss STATUS,Z
 goto hllPLBLA
 movfw margp2
 andlw .255-.16
 goto hllSLBLA
hllPLBLA:
 xorlw .4
 xorlw .5
 btfss STATUS,Z
 goto hllQLBLA
 movfw margp2
 andlw .255-.32
 goto hllSLBLA
hllQLBLA:
 xorlw .5
 xorlw .6
 btfss STATUS,Z
 goto hllRLBLA
 movfw margp2
 andlw .255-.64
 goto hllSLBLA
hllRLBLA:
 movfw margp2
 andlw .255-.128
hllSLBLA:
 PUSH
 return
#undefine margp2
ukrnl2 UDATA
stack res HLOE_STACK_SIZE
ukrnl3 UDATA
alt_stack res HLOE_STACK2_SIZE
ukernl udata
hllblss00 res 1
#ifdef HLLMULTITASK
#ifdef HLLGUARDhllblss
hllblss00isr res 1
#endif
#endif
hllblss01 res 1
#ifdef HLLMULTITASK
#ifdef HLLGUARDhllblss
hllblss01isr res 1
#endif
#endif
hllblss02 res 1
#ifdef HLLMULTITASK
#ifdef HLLGUARDhllblss
hllblss02isr res 1
#endif
#endif
ukrshr udata_shr
softstack0 res 1
#ifdef HLLMULTITASK
softstack0isr res 1
#endif
softstack1 res 1
#ifdef HLLMULTITASK
softstack1isr res 1
#endif
softstack2 res 1
#ifdef HLLMULTITASK
softstack2isr res 1
#endif
alt_fsr res 1
W_Save res 1
STATUS_Save res 1
FSR_Save res 1
ALT_Save res 1
in_isr res 1
hllkrna00 CODE
#define mterm hllblss00
mul:
 #ifdef HLLMULTITASK
 #ifdef HLLGUARDhllblss
 movf in_isr,f
 btfsc STATUS,Z
 goto mul0
 POP
 banksel mterm
 movwf mterm
 clrw
 addwf HLINDF,w
 decf mterm,f
 btfss STATUS,Z
 goto $-3
 movwf HLINDF
 return
#undefine mterm
#define mterm hllblss00+1
mul0:
#endif
#endif
 POP
 banksel mterm
 movwf mterm
 clrw
 addwf HLINDF,w
 decf mterm,f
 btfss STATUS,Z
 goto $-3
 movwf HLINDF
 return
#undefine mterm
hllkrna01 CODE
add:
 POP
 addwf HLINDF,w
 decf HLFSR,f
 PUSH
 return
hllkrna02 CODE
negti:
 comf HLINDF,f
 incf HLINDF,f
 return
hllkrna03 CODE
printch:
 POP
#ifdef HLLMULTITASK
 banksel PIR1
 btfss PIR1,TXIF
 goto $-1
#endif
 banksel TXREG
 movwf TXREG
 banksel PIR1
 btfss PIR1,TXIF
 goto $-1
 return
hllkrna04 CODE
#define mterm hllblss00
#define mdiv hllblss01
#define mquot hllblss02
modu:
#ifdef HLLMULTITASK
#ifdef HLLGUARDhllblss
 movf in_isr,f
 btfsc STATUS,Z
 goto modu0
 banksel mquot
 clrf mquot
 POP
 movwf mdiv
 POP
 movwf mterm
 movfw mdiv
 subwf mterm,f
 btfss STATUS,C
 goto $+3
 incf mquot,f
 goto $-4
 addwf mterm,w
 PUSH
 return
#undefine mterm
#undefine mdiv
#undefine mquot
#define mterm hllblss00+1
#define mdiv hllblss01+1
#define mquot hllblss02+1
modu0:
#endif
#endif
 banksel mquot
 clrf mquot
 POP
 movwf mdiv
 POP
 movwf mterm
 movfw mdiv
 subwf mterm,f
 btfss STATUS,C
 goto $+3
 incf mquot,f
 goto $-4
 addwf mterm,w
 PUSH
 return
#undefine mterm
#undefine mdiv
#undefine mquot
hllkrna05 CODE
#define mterm hllblss00
#define mdiv hllblss01
#define mquot hllblss02
divu:
#ifdef HLLMULTITASK
#ifdef HLLGUARDhllblss
 movf in_isr,f
 btfsc STATUS,Z
 goto divu0
 banksel mquot
 clrf mquot
 POP
 movwf mdiv
 POP
 movwf mterm
 movfw mdiv
 subwf mterm,f
 btfss STATUS,C
 goto $+3
 incf mquot,f
 goto $-4
 movfw mquot
 PUSH
 return
#undefine mterm
#undefine mdiv
#undefine mquot
#define mterm hllblss00+1
#define mdiv hllblss01+1
#define mquot hllblss02+1
divu0:
#endif
#endif
 banksel mquot
 clrf mquot
 POP
 movwf mdiv
 POP
 movwf mterm
 movfw mdiv
 subwf mterm,f
 btfss STATUS,C
 goto $+3
 incf mquot,f
 goto $-4
 movfw mquot
 PUSH
 return
#undefine mterm
#undefine mdiv
#undefine mquot
hllkrna06 CODE
getch:
 banksel PIR1
geth2:
 btfss PIR1,RCIF
 goto geth2
 banksel RCREG
 movf RCREG,w
 PUSH
 return
hllkrna07 CODE
eq:
 POP
 xorwf HLINDF,w
 movlw .1
 btfss STATUS,Z
 movlw .0
 decf HLFSR,f
 PUSH
 return
#define sstack0 softstack0
#define sstack1 softstack1
#define sstack2 softstack2
hllparmcore macro
 movwf sstack2
 movfw HLFSR
 movwf sstack0
 movfw HLINDF
 subwf sstack2,w
 movwf HLFSR
 movfw HLINDF
 movwf sstack1
 movfw sstack0
 movwf HLFSR
 clrf sstack0
 movfw sstack1
 movwf HLINDF
 movfw sstack2
 pagesel kpush
 goto kpush
 endm
hllkrna08 CODE
parm:
 KPOP
#ifdef HLLMULTITASK
 movf in_isr,f
 btfss STATUS,Z
 goto parm4
 hllparmcore
#undefine sstack0
#undefine sstack1
#undefine sstack2
#define sstack0 softstack0+1
#define sstack1 softstack1+1
#define sstack2 softstack2+1
parm4:
#endif
 hllparmcore
#undefine sstack0
#undefine sstack1
#undefine sstack2
hllkrna09 CODE
kpop:
 KPOP
 return
hllkrna10 CODE
kpush:
 KPUSH
 return
hllkrna11 CODE
dispose:
 DISCARD
 return
hlog1b CODE
#define margp2 hllblss00
setbit:
#ifdef HLLMULTITASK
#ifdef HLLGUARDhllblss
 movf in_isr,f
 btfsc STATUS,Z
 goto setbit0
 POP
 banksel margp2
 movwf  margp2
 POP
 xorlw .0
 btfss STATUS,Z
 goto tcf00h
 movfw  margp2
 iorlw .1
 goto tcf99h
tcf00h:
 xorlw .1
 btfss STATUS,Z
 goto tcf01h
 movfw  margp2
 iorlw .2
 goto tcf99h
tcf01h:
 xorlw .1
 xorlw .2
 btfss STATUS,Z
 goto tcf02h
 movfw margp2
 iorlw .4
 goto tcf99h
tcf02h:
 xorlw .2
 xorlw .3
 btfss STATUS,Z
 goto tcf03h
 movfw margp2
 iorlw .8
 goto tcf99h
tcf03h:
 xorlw .3
 xorlw .4
 btfss STATUS,Z
 goto tcf04h
 movfw margp2
 iorlw .16
 goto tcf99h
tcf04h:
 xorlw .4
 xorlw .5
 btfss STATUS,Z
 goto tcf05h
 movfw margp2
 iorlw .32
 goto tcf99h
tcf05h:
 xorlw .5
 xorlw .6
 btfss STATUS,Z
 goto tcf07h
 movfw margp2
 iorlw .64
 goto tcf99h
tcf07h:
 movfw margp2
 iorlw .128
tcf99h:
 PUSH
 return
#undefine margp2
#define margp2 hllblss00+1
setbit0:
#endif
#endif
 POP
 banksel margp2
 movwf  margp2
 POP
 xorlw .0
 btfss STATUS,Z
 goto hllTLBLA
 movfw  margp2
 iorlw .1
 goto hllABLBL
hllTLBLA:
 xorlw .1
 btfss STATUS,Z
 goto hllULBLA
 movfw  margp2
 iorlw .2
 goto hllABLBL
hllULBLA:
 xorlw .1
 xorlw .2
 btfss STATUS,Z
 goto hllVLBLA
 movfw margp2
 iorlw .4
 goto hllABLBL
hllVLBLA:
 xorlw .2
 xorlw .3
 btfss STATUS,Z
 goto hllWLBLA
 movfw margp2
 iorlw .8
 goto hllABLBL
hllWLBLA:
 xorlw .3
 xorlw .4
 btfss STATUS,Z
 goto hllXLBLA
 movfw margp2
 iorlw .16
 goto hllABLBL
hllXLBLA:
 xorlw .4
 xorlw .5
 btfss STATUS,Z
 goto hllYLBLA
 movfw margp2
 iorlw .32
 goto hllABLBL
hllYLBLA:
 xorlw .5
 xorlw .6
 btfss STATUS,Z
 goto hllZLBLA
 movfw margp2
 iorlw .64
 goto hllABLBL
hllZLBLA:
 movfw margp2
 iorlw .128
hllABLBL:
 PUSH
 return
#undefine margp2
sfpu UDATA
sfploc00 res .1
sfpaux00 res .1
sfploc01 res .1
sfpaux01 res .1
sfploc02 res .1
sfpaux02 res .1
sfploc03 res .1
sfpaux03 res .1
sfploc04 res .1
sfploc05 res .1
sfploc06 res .1
sfploc07 res .1
sfploc08 res .1
sfploc09 res .1
sfploc10 res .1
sfploc11 res .1
sfploc12 res .1
sfploc13 res .1
sfploc14 res .1
sfploc15 res .1
sfploc16 res .1
sfploc17 res .1
sfp_add CODE
#define mout_hi sfploc00
#define big_c sfploc01
#define min2_lower sfploc02
#define minres sfploc03
#define fulmin2H sfploc04
#define fulminH sfploc05
#define min_lower sfploc06
#define rev sfploc07
#define fulmin2 sfploc08
#define fulmin sfploc09
#define multiplier sfploc10
#define mout sfploc11
#define util sfploc12
#define neg sfploc13
#define ein2 sfploc14
#define min2 sfploc15
#define ein1 sfploc16
#define min sfploc17
addf:
 POP
 banksel min
 movwf ein2
 POP
 banksel min
 movwf min2
 POP
 banksel min
 movwf ein1
 POP
 banksel min
 movwf min
 bcf minres,0
 clrf mout
 clrf neg
 btfsc min,7
 bsf neg,0
 btfsc min2,7
 bsf neg,1
 btfsc ein2,7
 goto hllDBLBL
 btfss ein1,7
 goto hllBBLBL
 goto hllFBLBL
hllBBLBL:
 movfw ein1
 subwf ein2,w
 btfss STATUS,C
 goto hllEBLBL
hllCBLBL:
 goto hllFBLBL
hllDBLBL:
 btfsc ein1,7
 goto hllBBLBL
hllEBLBL:
 movf ein2,w
 xorwf ein1,f
 movf ein1,w
 xorwf ein2,f
 movf ein2,w
 xorwf ein1,f
 movf min2,w
 xorwf min,f
 movf min,w
 xorwf min2,f
 movf min2,w
 xorwf min,f
 bsf rev,0
 goto hllFBLBL_done
hllFBLBL:
 clrf rev
hllFBLBL_done:
 movfw min
 andlw .127
 movwf min
 movfw min2
 andlw .127
 movwf min2
 clrf fulminH
 movfw min
 addlw .128
 btfsc STATUS,C
 incf fulminH,f
 movwf fulmin
 clrf fulmin2H
 movfw min2
 addlw .128
 btfsc STATUS,C
 incf fulmin2H,f
 movwf fulmin2
 clrf min_lower
 clrf min2_lower
hllHBLBL:
 movfw ein1
 xorwf ein2,w
 btfsc STATUS,Z
 goto hllJBLBL
 bcf STATUS,C
 rrf min_lower,f
 bcf STATUS,C
 rrf fulmin,f
 btfss STATUS,C
 goto hllIBLBL
 bsf min_lower,7
hllIBLBL:
 incf ein1,f
 goto hllHBLBL
hllJBLBL:
 btfss rev,0
 goto hllKBLBL
 movf fulmin2,w
 xorwf fulmin,f
 movf fulmin,w
 xorwf fulmin2,f
 movf fulmin2,w
 xorwf fulmin,f
 movfw min_lower
 movwf min2_lower
 clrf min_lower
hllKBLBL:
 banksel min_lower
hllLBLBL:
 btfss neg,0
 goto hllMBLBL
 btfss neg,1
 goto hllMBLBL
 bsf minres,0
 goto hllDCLBL
hllMBLBL:
 btfss neg,0
 goto hllUBLBL
hllNBLBL:
 movfw fulmin
 subwf fulmin2,w
 movwf mout
 btfsc STATUS,C
 goto hllOBLBL
 bsf minres,0
 goto hllVBLBL
hllOBLBL:
 movf mout,f
 btfsc STATUS,Z
 goto hllJCLBL
hllPBLBL:
 btfsc mout,7
 goto hllRBLBL
 bcf STATUS,C
 rlf mout,f
 btfss min_lower,7
 goto hllQBLBL
 decf mout,f
 movlw .255
 xorwf mout,w
 btfss STATUS,Z
 goto hllQBLBL
 bsf minres,0
 goto hllVBLBL
hllQBLBL:
 bcf STATUS,C
 rlf min_lower,f
 decf ein1,f
 goto hllPBLBL
hllRBLBL:
 btfss min_lower,7
 goto hllTBLBL
 decf mout,f
 movlw .255
 xorwf mout,w
 btfss STATUS,Z
 goto hllSBLBL
 bsf minres,0
 goto hllVBLBL
hllSBLBL:
 movlw .127
 xorwf mout,w
 btfss STATUS,Z
 goto hllTBLBL
 bcf STATUS,C
 rlf mout,f
 decf ein1,f
 btfsc min_lower,6
 goto hllTBLBL
 incf mout,f
hllTBLBL:
 goto hllJCLBL
hllUBLBL:
 btfss neg,1
 goto hllDCLBL
hllVBLBL:
 movfw fulmin2
 subwf fulmin,w
 movwf mout
 btfsc STATUS,C
 goto hllWBLBL
 bsf minres,0
 goto hllNBLBL
hllWBLBL:
 movf mout,f
 btfsc STATUS,Z
 goto hllJCLBL
hllXBLBL:
 btfsc mout,7
 goto hllRBLBLb
 bcf STATUS,C
 rlf mout,f
 btfss min2_lower,7
 goto hllYBLBL
 decf mout,f
 movlw .255
 xorwf mout,w
 btfss STATUS,Z
 goto hllYBLBL
 bsf minres,0
 goto hllNBLBL
hllYBLBL:
 bcf STATUS,C
 rlf min2_lower,f
 decf ein1,f
 goto hllXBLBL
hllRBLBLb:
 btfss min2_lower,7
 goto hllBCLBL
 decf mout,f
 movlw .255
 xorwf mout,w
 btfss STATUS,Z
 goto hllACLBL
 bsf minres,0
 goto hllNBLBL
hllACLBL:
 movlw .127
 xorwf mout,w
 btfss STATUS,Z
 goto hllBCLBL
 bcf STATUS,C
 rlf mout,f
 decf ein1,f
 btfsc min2_lower,6
 goto hllBCLBL
 incf mout,f
hllBCLBL:
 goto hllJCLBL
hllCCLBL:
 movfw fulmin
 subwf fulmin2,w
 movwf mout
 bsf minres,0
 goto hllJCLBL
hllDCLBL:
 clrf mout_hi
 movf fulmin,w
 addwf fulmin2,w
 movwf mout
 btfss STATUS,C
 goto hllECLBL
 incf mout_hi,f
hllECLBL:
 movf min_lower,w
 addwf min2_lower,w
 btfsc STATUS,C
 goto hllFCLBL
 andlw .128
 btfss STATUS,Z
 goto hllFCLBL
 goto hllGCLBL
hllFCLBL:
 incf mout,f
 movf mout,f
 btfss STATUS,Z
 goto hllGCLBL
 incf mout_hi,f
hllGCLBL:
 movf mout_hi,f
 btfsc STATUS,Z
 goto hllJCLBL
 clrf big_c
 bcf STATUS,C
 rrf fulmin,f
 btfsc STATUS,C
 bsf big_c,0
 bcf STATUS,C
 rrf min_lower,f
 btfss big_c,0
 goto hllHCLBL
 bsf min_lower,7
hllHCLBL:
 clrf big_c
 bcf STATUS,C
 rrf fulmin2,f
 btfsc STATUS,C
 bsf big_c,0
 bcf STATUS,C
 rrf min2_lower,f
 btfss big_c,0
 goto hllICLBL
 bsf min_lower,7
hllICLBL:
 incf ein1,f
 goto hllDCLBL
hllJCLBL:
 movfw mout
 btfss STATUS,Z
 goto hllKCLBL
 movlw .0
 PUSH
 movlw 0x80
 PUSH
 goto hllLCLBL
hllKCLBL:
 movfw mout
 addlw -.128
 movwf mout
 btfsc minres,0
 bsf mout,7
 movfw mout
 PUSH
 banksel min
 movfw ein1
 PUSH
hllLCLBL:
 return
#undefine mout_hi
#undefine big_c
#undefine min2_lower
#undefine minres
#undefine fulmin2H
#undefine fulminH
#undefine min_lower
#undefine rev
#undefine fulmin2
#undefine fulmin
#undefine multiplier
#undefine mout
#undefine util
#undefine neg
#undefine ein2
#undefine min2
#undefine ein1
#undefine min
sfp_copy CODE
#define mye sfploc00
#define mym sfploc01
copyf:
 POP
 banksel mye
 movwf mye
 POP
 movwf mym
 PUSH
 movfw mye
 PUSH
 movfw mym
 PUSH
 movfw mye
 PUSH
 return
#undefine mye
#undefine mym
sfp_div CODE
#define rounded sfploc00
#define big_c sfploc01
#define a_hi_msb sfploc02
#define loop_count sfploc03
#define run_total sfploc04
#define min2_lower sfploc05
#define subtrahend sfploc06
#define quotient_lo sfploc07
#define term sfploc08
#define exam sfploc09
#define multiplier sfploc10
#define mout sfploc11
#define util sfploc12
#define neg sfploc13
#define ein2 sfploc14
#define min2 sfploc15
#define ein sfploc16
#define min sfploc17
divf:
 POP
 banksel min
 movwf ein2
 clrf neg
 POP
 banksel min
 movwf min2
 btfsc min2,7
 bsf neg,1
 bcf min2,7
 POP
 banksel min
 movwf ein
 POP
 banksel min
 movwf min
 btfsc min,7
 bsf neg,0
 bcf min,7
 movfw ein
 movwf util
 movfw ein2
 subwf util,f
 clrf exam
 movfw min
 addlw .128
 movwf min
 movfw min2
 addlw .128
 movwf min2
 movwf term
 clrf quotient_lo
 clrf subtrahend
 clrf min2_lower
 clrf run_total
 clrf loop_count
hllMCLBL:
 movf loop_count,w
 xorlw .16
 btfsc STATUS,Z
 goto hllRCLBL
 movf min,w
 andlw .128
 movwf a_hi_msb
 bcf STATUS,C
 rlf min2_lower,f
 bcf STATUS,C
 rlf run_total,f
 btfss STATUS,C
 goto hllNCLBL
 incf min2_lower,f
hllNCLBL:
 bcf STATUS,C
 rlf min,f
 bcf STATUS,C
 rlf quotient_lo,f
 btfsc STATUS,C
 incf min,f
 movf a_hi_msb,f
 btfsc STATUS,Z
 goto hllOCLBL
 incf run_total,f
 btfsc STATUS,Z
 incf min2_lower,f
hllOCLBL:
 clrf big_c
 movfw min2
 subwf run_total,w
 btfss STATUS,C
 bsf big_c,0
 movf subtrahend,w
 subwf min2_lower,f
 movf min2,w
 subwf run_total,f
 btfsc big_c,0
 decf min2_lower,f
 btfss min2_lower,7
 goto hllPCLBL
 movf min2,w
 addwf run_total,w
 clrf big_c
 btfsc STATUS,C
 bsf big_c,0
 movf subtrahend,w
 addwf min2_lower,f
 movf min2,w
 addwf run_total,f
 btfsc big_c,0
 incf min2_lower,f
 movlw .254
 andwf quotient_lo,f
 goto hllQCLBL
hllPCLBL:
 movlw .1
 iorwf quotient_lo,f
hllQCLBL:
 incf loop_count,f
 goto hllMCLBL
hllRCLBL:
 movf quotient_lo,w
 movwf mout
 clrf multiplier
 btfsc min,0
 bsf multiplier,0
 bcf STATUS,C
 rrf term,f
 clrf rounded
 movfw run_total
 subwf term,w
 btfsc STATUS,C
 goto hllSCLBL
 incf mout,f
 bsf rounded,0
hllSCLBL:
 movf multiplier,f
 btfsc STATUS,Z
 goto hllVCLBL
 btfss mout,0
 goto hllTCLBL
 btfsc rounded,0
 goto hllTCLBL
 bcf STATUS,C
 rrf mout,f
 incf mout,f
 goto hllUCLBL
hllTCLBL:
 bcf STATUS,C
 rrf mout,f
hllUCLBL:
 bsf mout,7
 goto hllWCLBL
hllVCLBL:
 decf util,f
hllWCLBL:
 movfw mout
hllXCLBL:
 addlw -.128
 movwf exam
 btfss exam,7
 goto hllYCLBL
 addlw .128
 decf util,f
 bcf STATUS,C
 rlf mout,f
 movfw mout
 goto hllXCLBL
hllYCLBL:
 movwf mout
 btfsc neg,0
 goto hllZCLBL
 btfss neg,1
 goto hllBDLBL
 goto hllADLBL
hllZCLBL:
 btfsc neg,1
 goto hllBDLBL
hllADLBL:
 movfw mout
 iorlw .128
 movwf mout
hllBDLBL:
 movfw mout
 PUSH
 banksel min
 movfw util
 PUSH
 return
#undefine rounded
#undefine big_c
#undefine a_hi_msb
#undefine loop_count
#undefine run_total
#undefine min2_lower
#undefine subtrahend
#undefine quotient_lo
#undefine term
#undefine exam
#undefine multiplier
#undefine mout
#undefine util
#undefine neg
#undefine ein2
#undefine min2
#undefine ein
#undefine min
sfp_tou CODE
#define util sfploc00
#define divisor sfploc01
#define karg2 sfploc02
#define karg1 sfploc03
ftou:
 POP
 banksel karg2
 movwf karg2
 clrf karg1
 btfss karg2,7
 goto hllCDLBL
 POP
 goto hllMDLBL
hllCDLBL:
 POP
 addlw .128
 banksel karg1
 movwf karg1
 movfw karg2
 sublw .7
 movwf divisor
 movlw high hllDDLBL
 movwf PCLATH
 movlw low hllDDLBL
 addwf divisor,w
 btfsc STATUS,C
 incf PCLATH,f
 movwf PCL
hllDDLBL:
 goto hllLDLBL
 goto hllKDLBL
 goto hllJDLBL
 goto hllIDLBL
 goto hllHDLBL
 goto hllGDLBL
 goto hllFDLBL
 goto hllEDLBL
hllEDLBL:
 bcf STATUS,C
 rrf karg1,f
 rrf util,f
hllFDLBL:
 bcf STATUS,C
 rrf karg1,f
 rrf util,f
hllGDLBL:
 bcf STATUS,C
 rrf karg1,f
 rrf util,f
hllHDLBL:
 bcf STATUS,C
 rrf karg1,f
 rrf util,f
hllIDLBL:
 bcf STATUS,C
 rrf karg1,f
 rrf util,f
hllJDLBL:
 bcf STATUS,C
 rrf karg1,f
 rrf util,f
hllKDLBL:
 bcf STATUS,C
 rrf karg1,f
 rrf util,f
hllLDLBL:
hllMDLBL:
 banksel karg1
 movfw karg1
 PUSH
 return
#undefine util
#undefine divisor
#undefine karg2
#undefine karg1
sfp_gt CODE
#define karg4 sfploc00
#define karg3 sfploc01
#define karg2 sfploc02
#define karg1 sfploc03
#define neg2 sfploc04
#define neg sfploc05
gtf:
 POP
 banksel karg4
 movwf karg4
 POP
 movwf karg3
 POP
 movwf karg2
 POP
 movwf karg1
 clrf neg
 clrf neg2
 btfsc karg1,7
 bsf neg,0
 btfsc karg3,7
 bsf neg2,0
 btfss neg,0
 goto hllNDLBL
 btfsc neg2,0
 goto hllNDLBL
 movlw .0
 PUSH
 goto hllXDLBL
hllNDLBL:
 btfsc neg,0
 goto hllODLBL
 btfss neg2,0
 goto hllODLBL
 movlw .1
 PUSH
 goto hllXDLBL
hllODLBL:
 bcf karg1,7
 bcf karg3,7
 btfsc karg4,7
 goto hllPDLBL
 movf karg4,f
 btfsc STATUS,Z
 goto hllPDLBL
 movf karg2,f
 btfsc STATUS,Z
 goto hllRDLBL
 btfsc karg2,7
 goto hllRDLBL
 goto hllQDLBL
hllPDLBL:
 btfsc karg2,7
 goto hllRDLBL
 movf karg2,f
 btfsc STATUS,Z
 goto hllRDLBL
 goto hllWDLBL
hllQDLBL:
 movfw  karg2
 subwf karg4,w
 btfsc STATUS,C
 goto hllRDLBL
 goto hllWDLBL
hllRDLBL:
 btfsc karg2,7
 goto hllSDLBL
 movf karg2,f
 btfsc STATUS,Z
 goto hllSDLBL
 movf karg4,f
 btfsc STATUS,Z
 goto hllUDLBL
 btfsc karg4,7
 goto hllUDLBL
 goto hllTDLBL
hllSDLBL:
 btfsc karg4,7
 goto hllUDLBL
 movf karg4,f
 btfsc STATUS,Z
 goto hllUDLBL
 goto hllVDLBL
hllTDLBL:
 movfw  karg4
 subwf karg2,w
 btfsc STATUS,C
 goto hllUDLBL
 goto hllVDLBL
hllUDLBL:
 movfw karg1
 subwf karg3,w
 btfss STATUS,C
 goto hllWDLBL
 movfw karg3
 subwf karg1,w
 btfss STATUS,C
 goto hllVDLBL
 movlw .0
 PUSH
 goto hllXDLBL
hllVDLBL:
 banksel neg
 movlw .0
 btfsc neg,0
 movlw .1
 PUSH
 goto hllXDLBL
hllWDLBL:
 banksel neg
 movlw .0
 btfss neg,0
 movlw .1
 PUSH
hllXDLBL:
 return
#undefine karg4
#undefine karg3
#undefine karg2
#undefine karg1
#undefine neg2
#undefine neg
sfp_iszero CODE
iszerof:
 POP
 xorlw 0x80
 btfss STATUS,Z
 goto nonzf
 POP
 andlw .127
 xorlw .0
 btfss STATUS,Z
 goto nonzg
 movlw .1
 goto nonzr
nonzf:
 decf HLFSR,f
nonzg:
 movlw .0
nonzr:
 PUSH
 return
sfp_mul CODE
#define savelsb sfploc00
#define iterator sfploc01
#define hi_byte sfploc02
#define make_mout sfploc03
#define factor2 sfploc04
#define factor1 sfploc05
#define mulhi sfploc06
#define mout sfploc07
#define util sfploc08
#define neg sfploc09
#define ein2 sfploc10
#define min2 sfploc11
#define ein sfploc12
#define min sfploc13
mulf:
 POP
 banksel ein2
 movwf ein2
 xorlw .128
 btfss STATUS,Z
 goto hllYDLBL
 movlw .127
 andwf HLINDF,w
 btfsc STATUS,Z
 goto hllGELBL
hllYDLBL:
 clrf neg
 POP
 banksel min2
 movwf min2
 btfsc min2,7
 bsf neg,1
 bcf min2,7
 POP
 banksel ein
 movwf ein
 xorlw .128
 btfss STATUS,Z
 goto hllZDLBL
 movlw .127
 andwf HLINDF,w
 btfsc STATUS,Z
 goto hllHELBL
hllZDLBL:
 POP
 banksel min
 movwf min
 btfsc min,7
 bsf neg,0
 bcf min,7
 movfw ein
 movwf util
 movfw ein2
 addwf util,f
 movfw min
 addlw .128
 PUSH
 banksel min2
 movfw min2
 addlw .128
 PUSH
 goto hllJELBL
hllAELBL:
 POP
 banksel mout
 movwf mout
 btfss mout,7
 goto hllBELBL
 POP
 banksel util
 incf util,f
 andlw .128
 btfss STATUS,Z
 incf mout,f
 goto hllCELBL
hllBELBL:
 bcf STATUS,C
 POP
 banksel savelsb
 movwf savelsb
 andlw .128
 btfss STATUS,Z
 bsf STATUS,C
 banksel mout
 rlf mout,f
 movf savelsb,w
 banksel mout
 andlw .64
 btfss STATUS,Z
 incf mout,f
hllCELBL:
 movfw mout
 addlw -.128
 movwf mout
 btfsc neg,0
 goto hllDELBL
 btfss neg,1
 goto hllFELBL
 goto hllEELBL
hllDELBL:
 btfsc neg,1
 goto hllFELBL
hllEELBL:
 movfw mout
 iorlw .128
 movwf mout
hllFELBL:
 movfw mout
 PUSH
 banksel util
 movfw util
 PUSH
 goto hllIELBL
hllGELBL:
 POP
 POP
hllHELBL:
 POP
 movlw .128
 PUSH
 PUSH
hllIELBL:
 return
hllJELBL:
 POP
 banksel factor1
 movwf factor1
 POP
 banksel factor2
 movwf factor2
 clrf hi_byte
 clrf make_mout
 clrf iterator
hllKELBL:
 btfss factor1,0
 goto hllLELBL
 movfw factor2
 addwf make_mout,f
 btfsc STATUS,C
 incf hi_byte,f
hllLELBL:
 bcf STATUS,C
 rrf factor1,f
 bcf STATUS,C
 rrf hi_byte,f
 rrf make_mout,f
 btfsc STATUS,C
 bsf factor1,7
 incf iterator,f
 movlw .8
 xorwf iterator,w
 btfss STATUS,Z
 goto hllKELBL
 movfw factor1
 PUSH
 banksel make_mout
 movfw make_mout
 PUSH
 goto hllAELBL
#undefine savelsb
#undefine iterator
#undefine hi_byte
#undefine make_mout
#undefine factor2
#undefine factor1
#undefine mulhi
#undefine mout
#undefine util
#undefine neg
#undefine ein2
#undefine min2
#undefine ein
#undefine min
sfp_uto CODE
#define arg1 sfploc00
utof:
 POP
 banksel arg1
 movwf arg1
 btfss arg1,7
 goto utfdiv2
 bcf arg1,7
 movfw arg1
 PUSH
 movlw .7
 PUSH
 goto utfdone
utfdiv2:
 btfss arg1,6
 goto utfdiv4
 banksel arg1
 bcf arg1,6
 bcf STATUS,C
 rlf arg1,f
 movfw arg1
 PUSH
 movlw .6
 PUSH
 goto utfdone
utfdiv4:
 btfss arg1,5
 goto utfdiv8
 banksel arg1
 bcf arg1,5
 bcf STATUS,C
 rlf arg1,f
 rlf arg1,f
 movfw arg1
 PUSH
 movlw .5
 PUSH
 goto utfdone
utfdiv8:
 btfss arg1,4
 goto utfdiv16
 banksel arg1
 bcf arg1,4
 bcf STATUS,C
 rlf arg1,f
 rlf arg1,f
 rlf arg1,f
 movfw arg1
 PUSH
 movlw .4
 PUSH
 goto utfdone
utfdiv16:
 btfss arg1,3
 goto utfdiv32
 banksel arg1
 bcf arg1,3
 bcf STATUS,C
 rlf arg1,f
 rlf arg1,f
 rlf arg1,f
 rlf arg1,f
 movfw arg1
 PUSH
 movlw .3
 PUSH
 goto utfdone
utfdiv32:
 btfss arg1,2
 goto utfdiv64
 banksel arg1
 bcf arg1,2
 bcf STATUS,C
 rlf arg1,f
 rlf arg1,f
 rlf arg1,f
 rlf arg1,f
 rlf arg1,f
 movfw arg1
 PUSH
 movlw .2
 PUSH
 goto utfdone
utfdiv64:
 btfss arg1,1
 goto utfone
 banksel arg1
 bcf arg1,1
 bcf STATUS,C
 rlf arg1,f
 rlf arg1,f
 rlf arg1,f
 rlf arg1,f
 rlf arg1,f
 rlf arg1,f
 movfw arg1
 PUSH
 movlw .1
 PUSH
 goto utfdone
utfone:
 btfss arg1,0
 goto utfzero
 movlw .0
 PUSH
 movlw .0
 PUSH
 goto utfdone
utfzero:
 movlw .0
 PUSH
 movlw -.128
 PUSH
utfdone:
 return
#undefine arg1
 end
